#
# OOoHtml
#
# (c) Copyright 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#
# Released unter the terms of the Mozilla Public License (MPL) v1.1
#
# Please see README.txt for further information
#

"""
OpenOffice.org documents to HTML converter

$Id$
"""

import sys, os
from fnmatch import fnmatch
from zipfile import ZipFile

import xml.dom.minidom
from xml.xslt.Processor import Processor

#XXX I wish I wouldn't have to make it dependent on Zope here
from Globals import package_home


proper_extension = {
    "text":         "sxw",
    "spreadsheet":  "sxc",
    "presentation": "sxi",
    "drawing":      "sxd"
    }


class OOoHtml:
    """
    OpenOffice.org documents to HTML converter
    """

    def __init__( self, ooofile ):
        self._zip = ZipFile( ooofile )


    def getFile( self, name ):
        """
        Return a file from the archive as data stream
        """
        return self._zip.read( name )


    def getDocType( self ):
        """
        Return the type of document (writer, spreadsheet, etc.)
        """
        content_xml = self.getFile( "content.xml" )
        dom = xml.dom.minidom.parseString( content_xml )
        document_content = dom.getElementsByTagName( "office:document-content" )[0]
        return document_content.getAttribute("office:class")


    def getMetaData( self ):
        """
        Return the type of document (writer, spreadsheet, etc.)
        """
        meta = {}
        meta_xml = self.getFile( "meta.xml" )
        dom = xml.dom.minidom.parseString( meta_xml )

        # Dublin Core properties
        for info in ( "title", "subject", "language" ):
            elements = dom.getElementsByTagName( "dc:"+info )
            meta[info] = ""
            if len(elements) and len(elements[0].childNodes):
                meta[info] = elements[0].childNodes[0].data

        # other meta data
        for info in ( "creation-date", "editing-cycles", "editing-duration" ):
            elements = dom.getElementsByTagName( "meta:"+info )
            meta[info] = ""
            if len(elements) and len(elements[0].childNodes):
                meta[info] = elements[0].childNodes[0].data

        # keywords
        keywords = []
        for keyword in dom.getElementsByTagName( "meta:keyword" ):
            keywords.append( keyword.childNodes[0].data )
        meta["keywords"] = " ".join( keywords )

        # user defined
        user_defined = dom.getElementsByTagName( "meta:user-defined" )
        user_defined_list = []
        for element in user_defined:
            value = ""
            if len(element.childNodes):
                value = element.childNodes[0].data
            user_defined_list.append( { "name": element.getAttribute("meta:name"),
                                        "value": value   } )
        meta["user-defined"] = user_defined_list
        
        # statistics
        statistics = dom.getElementsByTagName( "meta:document-statistic" )[0]
        meta["statistics"] = {
            "table-count":     statistics.getAttribute("meta:table-count"),
            "image-count":     statistics.getAttribute("meta:image-count"),
            "object-count":    statistics.getAttribute("meta:object-count"),
            "page-count":      statistics.getAttribute("meta:page-count"),
            "paragraph-count": statistics.getAttribute("meta:paragraph-count"),
            "word-count":      statistics.getAttribute("meta:word-count"),
            "character-count": statistics.getAttribute("meta:character-count"),
            "cell-count":      statistics.getAttribute("meta:cell-count")
            }
        
        return meta


    def getHtml( self ):
        """
        Processes the XML content and returns HTML as string
        """
        content_xml = self.getFile( "content.xml" )
        proc = Processor()
        uri = os.path.join( package_home(globals()), "ooo2html.xsl" ) #XXX I wish I could do this differently
        proc.appendStylesheetUri( uri )
        return proc.runString( content_xml )


    def getPictures( self ):
        """
        Returns a dictionary with filenames as keys and the picture data stream as values
        """
        pictures = {}
        filelist = self._zip.namelist()
        pattern = "Pictures/*"
        filelist = [n for n in filelist if fnmatch( n, pattern )]
        for file in filelist:
            pictures[file[9:]] = self.getFile( file )
        return pictures

    


